<?php
declare(strict_types=1);

namespace GamesShop\Entities\Account;

use Doctrine\ORM\Mapping as ORM;
use GamesShop\Login\LoginMethod;
use GamesShop\Login\UserPermission;
use Symfony\Component\Uid\Uuid;
use Symfony\Component\Uid\UuidV4;

#[ORM\Entity]
#[ORM\Table(name: "users")]
class User
{

    #[ORM\Id()]
    #[ORM\Column]
    #[ORM\GeneratedValue]
    private int|null $id = null;
    #[ORM\Column(type: 'integer', enumType: LoginMethod::class)]
    private LoginMethod $loginMethod;
    #[ORM\Column]
    private string|null $foreignLoginId = null;
    #[ORM\Column]
    private string $name;
    #[ORM\Column]
    private string $profilePictureUrl;
    #[ORM\Column]
    private UserPermission $permission;

    public function __construct(LoginMethod $loginMethod, ?string $foreignLoginId, string $name, string $profilePictureUrl, UserPermission $permission)
    {
        $this->loginMethod = $loginMethod;
        $this->foreignLoginId = $foreignLoginId;
        $this->name = $name;
        $this->profilePictureUrl = $profilePictureUrl;
        $this->permission = $permission;
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getLoginMethod(): LoginMethod
    {
        return $this->loginMethod;
    }

    public function getForeignLoginId(): ?string
    {
        return $this->foreignLoginId;
    }

    public function getName(): string
    {
        return $this->name;
    }

    public function getProfilePictureUrl(): string
    {
        return $this->profilePictureUrl;
    }

    public function getPermission(): UserPermission
    {
        return $this->permission;
    }

    public function setName(string $name): void
    {
        $this->name = $name;
    }

    public function setProfilePictureUrl(string $profilePictureUrl): void
    {
        $this->profilePictureUrl = $profilePictureUrl;
    }

    public function setPermission(UserPermission $permission): void
    {
        $this->permission = $permission;
    }
}