<?php
declare(strict_types=1);

namespace GamesShop\Login;

use Doctrine\ORM\EntityManager;
use Exception;
use GamesShop\ContainerHandler;
use GamesShop\Entities\Account\User;

final class LoginHandler
{
    /**
     * @return class-string[]
 */
    private static array $providers;

    public function __construct(
        private readonly EntityManager $entityManager
    )
    {
    }

    public function isLoggedIn(): bool {
        $this->ensureSession();

        return isset($_SESSION['accountid']);
    }

    /**
     * @throws Exception
     */
    public function getLoginProvider(string $method): LoginProvider {
        $providers = self::getProviders();
        if (!array_key_exists($method, $providers)) {
            throw new Exception("Couldn't find method for login '{$method}'");
        }

        return ContainerHandler::get($providers[$method]);
    }

    public function setCurrentUser(User $user) {
        $this->ensureSession();

        $_SESSION['accountid'] = $user->getId();
    }

    public function getCurrentUser(): User {
        $this->ensureSession();

        $userid = $_SESSION['accountid'];
        return $this->entityManager->getRepository(User::class)->find($userid);
    }

    public function deleteSession() {
        $this->ensureSession();
        session_destroy();
    }

    private function ensureSession()
    {
        if (session_status() !== PHP_SESSION_ACTIVE) {
            session_start();
        }
    }

    /**
     * @return class-string[]
     */
    private static function getProviders(): array
    {
        return self::$providers ??= [
            'discord' => DiscordLoginProvider::class
        ];
    }
}